require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xhotfix',
  dscmeta_resource_friendly_name: 'xHotfix',
  dscmeta_resource_name: 'MSFT_xWindowsUpdate',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWindowsUpdate',
  dscmeta_module_version: '2.8.0.0',
  docs: 'The DSC xHotfix resource type.
         Automatically generated from version 2.8.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_id: {
      type: 'String',
      desc: 'Specifies the Hotfix ID.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Specifies whether the hotfix needs to be installed or uninstalled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_log: {
      type: 'Optional[String]',
      desc: 'Specifies the location of the log that contains information from the installation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the credential to use to authenticate to a UNC share if the path is on a UNC share.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_path: {
      type: 'String',
      desc: 'Specifies the path that contains the msu file for the hotfix installation.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
