require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwindowsupdateagent',
  dscmeta_resource_friendly_name: 'xWindowsUpdateAgent',
  dscmeta_resource_name: 'MSFT_xWindowsUpdateAgent',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xWindowsUpdate',
  dscmeta_module_version: '2.8.0.0',
  docs: 'The DSC xWindowsUpdateAgent resource type.
         Automatically generated from version 2.8.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_notifications: {
      type: "Optional[Enum['Disabled', 'ScheduledInstallation']]",
      desc: 'Indicates if Windows update agent should notify about updates, see:https://msdn.microsoft.com/en-us/library/windows/desktop/aa385806%28v=vs.85%29.aspx?f=255&MSPPError=-2147217396.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_source: {
      type: "Enum['WindowsUpdate', 'MicrosoftUpdate', 'WSUS']",
      desc: 'Indicates which source service Windows update agent should use.  Note: WSUS is not implemented in this resource.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_updatenow: {
      type: 'Optional[Boolean]',
      desc: 'Indicates if the resource should trigger an update during consistency.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_category: {
      type: "Optional[Array[Enum['Security', 'Important', 'Optional']]]",
      desc: 'Indicates if the resource should ensure all Windows Updates are installed or if Automatic updates should be disabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_rebootrequired: {
      type: 'Optional[Boolean]',
      desc: 'Indicates if Wua Requires a reboot.  Only returned if UpdateNow is specified.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_totalupdatesnotinstalled: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Count of updates not installed.  Only returned if UpdateNow is specified.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_automaticupdatesnotificationsetting: {
      type: 'Optional[String]',
      desc: 'Automatic Updates Notification Setting',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_issingleinstance: {
      type: "Enum['Yes']",
      desc: "Specifies the resource is a single instance, the value must be 'Yes'",
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
